@@ -32,7 +32,12 @@ impl<K: Hash + Ord> SearchIndex<K> {
     /// Basic usage:
     ///
     /// ```ignore
-    /// # use indicium::simple::{AutocompleteType, Indexable, SearchIndex, SearchType};
+    /// # use indicium::simple::{
+    /// #   AutocompleteType,
+    /// #   Indexable,
+    /// #   SearchIndex,
+    /// #   SearchType
+    /// # };
     /// #
     /// # struct MyStruct {
     /// #   title: String,
@@ -118,37 +123,9 @@ impl<K: Hash + Ord> SearchIndex<K> {
         // autocompleting:
         if let Some(last_keyword) = keywords.pop() {
 
-            // Perform `And` search for entire string without the last keyword.
-            // We also must convert the return `HashSet` into a `BTreeSet`. It
-            // should be investigated if `internal_search_and` should returning
-            // a `BTreeSet` instead.
-            let search_results: BTreeSet<&K> =
-                self.internal_search_and(keywords.as_slice())
-                    // Iterate over each key:
-                    .iter()
-                    // Copy each `&K` key reference from the iterator or we'll
-                    // get a doubly-referenced `&&K` key:
-                    .cloned()
-                    // Collect serach results into our `BTreeSet`:
-                    .collect();
-
-            // Get all autocomplete options for the last keyword and its keys:
-            let autocomplete_options: BTreeSet<&BTreeSet<K>> =
-                self.internal_autocomplete_keyword(&last_keyword)
-                    // Iterate over each search result:
-                    .iter()
-                    // Only keep this autocompletion if hasn't already been used
-                    // as a keyword:
-                    .filter(|(keyword, _keys)| !keywords.contains(keyword))
-                    // We're not interested in the `keyword` since we're
-                    // returning `&K` keys. Return only `&K` from the tuple:
-                    .map(|(_keyword, keys)| *keys)
-                    // Collect search results from each autocomplete option:
-                    .collect();
-
             // How we combine `search_results` and `autocomplete_options`
             // together depends on how many keywords there are in the search
-            // string. Strings that have only a single keyword, and a strings
+            // string. Strings that have only a single keyword, and strings
             // that have multiple keywords must be handled differently:
 
             match keywords.len() {
@@ -159,24 +136,33 @@ impl<K: Hash + Ord> SearchIndex<K> {
                 // `tribble` may be given.
                 //
                 // There are no previous keywords to intersect with, just the
-                // letter `t`. If we attempt to intersect with an empty
-                // `search_results`, no keys will ever be returned. So we must
-                // handle this scenario differently. We will return the keys for
-                // these autocomplete options without further processing:
+                // autocomplete options for the letter `t`. If we attempt to
+                // intersect this with an empty `search_results`, no keys will
+                // ever be returned. So we must handle this scenario
+                // differently. We will return the keys for these autocomplete
+                // options without further processing:
 
-                0 => autocomplete_options
-                    // Iterate over each autocomplete option:
-                    .iter()
-                    // Copy the `&BTreeSet` reference from the iterator or we'll
-                    // be working with a doubly-referenced `&&BTreeSet`:
-                    .cloned()
-                    // Flatten the `key` results from each autocomplete option
-                    // into our collection:
-                    .flatten()
+                0 => self.b_tree_map
+                    // Get matching keywords starting with (partial) keyword
+                    // string:
+                    .range(last_keyword.to_owned()..)
+                    // We did not specify an end bound for our `range` function
+                    // (see above.) `range` will return _every_ keyword greater
+                    // than the supplied keyword. The below `take_while` will
+                    // effectively break iteration when we reach a keyword that
+                    // does not start with our supplied (partial) keyword.
+                    .take_while(|(keyword, _keys)|
+                        keyword.starts_with(&last_keyword)
+                    )
                     // Only return `maximum_search_results` number of keys:
                     .take(*maximum_search_results)
-                    // And collect each key into a `BTreeSet` that will be the
-                    // search results.
+                    // We're not interested in the `keyword` since we're
+                    // returning `&K` keys. Return only `&K` from the tuple:
+                    .map(|(_keyword, keys)| keys)
+                    // Flatten the `BTreeSet<K>` from each autocomplete keyword
+                    // option into our collection:
+                    .flatten()
+                    // Collect all keyword autocompletions into a `Vec`:
                     .collect(),
 
                 // Consider this example search string: `Shatner t`.
@@ -188,36 +174,52 @@ impl<K: Hash + Ord> SearchIndex<K> {
                 // This match arm will intersect the results from each
                 // autocomplete option with `Shatner`. For both `trouble` and
                 // `tribble` autocomplete options, only keys that also exist for
-                // `Shatner` will be returned. All resulting keys for both
-                // autocomplete options will be flattened together:
+                // `Shatner` will be returned:
 
-                _ => autocomplete_options
-                    // Iterate over each autocomplete option:
-                    .iter()
-                    // For each autocomplete option, we will intersect its
-                    // search results with the search results of the preceding
-                    // keywords:
-                    .map(|autocompletion_keys| autocompletion_keys
-                        // Iterate over each key returned for this autocomplete
-                        // option:
-                        .iter()
-                        // Only keep the `&K` key for this autocomplete option
-                        // if it is contained in the search results for the
-                        // preceding keywords:
-                        .filter(|autocompletion_key|
-                            search_results.contains(autocompletion_key)
-                        ) // filter
-                        // Collect all resulting keys into a `BTreeSet`:
-                        .collect::<BTreeSet<&K>>()
-                    ) // map
-                    // Flatten the `key` results for each autocomplete option
-                    // into our collection:
-                    .flatten()
-                    // Only return `maximum_search_results` number of keys:
-                    .take(*maximum_search_results)
-                    // And collect each key into a `BTreeSet` that will be the
-                    // search results.
-                    .collect(),
+                _ => {
+
+                    // Perform `And` search for entire string, excluding the
+                    // last (partial) keyword:
+                    let search_results: BTreeSet<&K> =
+                        self.internal_search_and(keywords.as_slice())
+                            // Iterate over each key:
+                            .iter()
+                            // Copy each `&K` key reference from the iterator or
+                            // we'll get a doubly-referenced `&&K` key:
+                            .cloned()
+                            // Collect serach results into our `BTreeSet`:
+                            .collect();
+
+                    // Get keys for the last (partial) keyword:
+                    self.b_tree_map
+                        // Get matching keywords starting with (partial) keyword
+                        // string:
+                        .range(last_keyword.to_owned()..)
+                        // We did not specify an end bound for our `range`
+                        // function (see above.) `range` will return _every_
+                        // keyword greater than the supplied keyword. The below
+                        // `take_while` will effectively break iteration when we
+                        // reach a keyword that does not start with our supplied
+                        // (partial) keyword.
+                        .take_while(|(keyword, _keys)|
+                            keyword.starts_with(&last_keyword)
+                        )
+                        // We're not interested in the `keyword` since we're
+                        // returning `&K` keys. Return only `&K` from the tuple:
+                        .map(|(_keyword, keys)| keys)
+                        // Flatten the `BTreeSet<K>` from each autocomplete
+                        // keyword option into individual `K` keys:
+                        .flatten()
+                        // Intersect the key results from the autocomplete
+                        // options (produced from this iterator) with the search
+                        // results produced above:
+                        .filter(|key| search_results.contains(key))
+                        // Only return `maximum_search_results` number of keys:
+                        .take(*maximum_search_results)
+                        // Collect all keyword autocompletions into a `Vec`:
+                        .collect()
+
+                }, // _
 
             } // match
 
